/**
 * Copyright (c) 2010 Perforce Software. All rights reserved.
 */
package com.perforce.maven.scm.provider.p4;

import org.apache.maven.scm.CommandParameters;
import org.apache.maven.scm.ScmException;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.command.add.AddScmResult;
import org.apache.maven.scm.command.blame.BlameScmResult;
import org.apache.maven.scm.command.branch.BranchScmResult;
import org.apache.maven.scm.command.changelog.ChangeLogScmResult;
import org.apache.maven.scm.command.checkin.CheckInScmResult;
import org.apache.maven.scm.command.checkout.CheckOutScmResult;
import org.apache.maven.scm.command.diff.DiffScmResult;
import org.apache.maven.scm.command.edit.EditScmResult;
import org.apache.maven.scm.command.export.ExportScmResult;
import org.apache.maven.scm.command.list.ListScmResult;
import org.apache.maven.scm.command.login.LoginScmResult;
import org.apache.maven.scm.command.remove.RemoveScmResult;
import org.apache.maven.scm.command.status.StatusScmResult;
import org.apache.maven.scm.command.tag.TagScmResult;
import org.apache.maven.scm.command.unedit.UnEditScmResult;
import org.apache.maven.scm.command.update.UpdateScmResult;
import org.apache.maven.scm.provider.AbstractScmProvider;
import org.apache.maven.scm.provider.ScmProviderRepository;
import org.apache.maven.scm.repository.ScmRepositoryException;
import org.codehaus.plexus.util.StringUtils;

import com.perforce.maven.scm.provider.p4.command.add.P4AddCommand;
import com.perforce.maven.scm.provider.p4.command.blame.P4BlameCommand;
import com.perforce.maven.scm.provider.p4.command.branch.P4BranchCommand;
import com.perforce.maven.scm.provider.p4.command.changelog.P4ChangeLogCommand;
import com.perforce.maven.scm.provider.p4.command.checkin.P4CheckInCommand;
import com.perforce.maven.scm.provider.p4.command.checkout.P4CheckOutCommand;
import com.perforce.maven.scm.provider.p4.command.diff.P4DiffCommand;
import com.perforce.maven.scm.provider.p4.command.edit.P4EditCommand;
import com.perforce.maven.scm.provider.p4.command.export.P4ExportCommand;
import com.perforce.maven.scm.provider.p4.command.list.P4ListCommand;
import com.perforce.maven.scm.provider.p4.command.login.P4LoginCommand;
import com.perforce.maven.scm.provider.p4.command.remove.P4RemoveCommand;
import com.perforce.maven.scm.provider.p4.command.status.P4StatusCommand;
import com.perforce.maven.scm.provider.p4.command.tag.P4TagCommand;
import com.perforce.maven.scm.provider.p4.command.unedit.P4UnEditCommand;
import com.perforce.maven.scm.provider.p4.command.update.P4UpdateCommand;
import com.perforce.maven.scm.provider.p4.repository.P4ScmProviderRepository;

/**
 * This class validates the SCM URL, populate the ScmProviderRepository and
 * provide all commands supported by the P4 provider.
 *
 * @plexus.component role="org.apache.maven.scm.provider.ScmProvider"
 *                   role-hint="p4"
 */
public class P4ScmProvider extends AbstractScmProvider {

    /**
     * @see org.apache.maven.scm.provider.ScmProvider#makeProviderScmRepository(java
     *      .lang.String, char)
     */
    public ScmProviderRepository makeProviderScmRepository(
            String scmSpecificUrl, char delimiter)
            throws ScmRepositoryException {
        String host = "";
        int port = 0;
        String path = null;

        int i1 = scmSpecificUrl.indexOf(delimiter);
        int i2 = scmSpecificUrl.indexOf(delimiter, i1 + 1);

        if (i1 > 0) {
            int lastDelimiter = scmSpecificUrl.lastIndexOf(delimiter);
            if (lastDelimiter > 0) {
                path = scmSpecificUrl.substring(lastDelimiter + 1);
            }
            host = scmSpecificUrl.substring(0, i1);

            // If there is tree parts in the scm url, the second is the port
            if (i2 >= 0) {
                try {
                    if (lastDelimiter > (i1+1)) {
                        String tmp = scmSpecificUrl
                                .substring(i1 + 1, lastDelimiter);
                        port = Integer.parseInt(tmp);
                    }
                } catch (NumberFormatException ex) {
                    throw new ScmRepositoryException(
                            "The port has to be a number.");
                }
            }
        } else {
            path = scmSpecificUrl;
        }

        String user = null;
        String password = null;
        if (host != null && host.indexOf("@") > 1) {
            user = host.substring(0, host.indexOf("@"));
            host = host.substring(host.indexOf("@") + 1);
        }

        if (path != null && path.indexOf("@") > 1) {
            if (host != null) {
                if (getLogger().isWarnEnabled()) {
                    getLogger()
                            .warn("Username as part of path is deprecated, the new format is "
                                    + "scm:p4:[username@]host:port:path_to_repository");
                }
            }

            user = path.substring(0, path.indexOf("@"));
            path = path.substring(path.indexOf("@") + 1);
        }

        // Get the host and port from the environment variable 'P4PORT'
        String envHost = "localhost";
        int envPort = 1666;
        String p4Port = System.getenv("P4PORT");
        if (StringUtils.isNotEmpty(p4Port)) {
            String[] p4parts = p4Port.split(":");
            if (p4parts.length > 0) {
                if (StringUtils.isNotEmpty(p4parts[0])) {
                    envHost = p4parts[0];
                }
            }
            if (p4parts.length > 1) {
                if (StringUtils.isNotEmpty(p4parts[1])) {
                    try {
                        envPort = Integer.parseInt(p4parts[1]);
                    } catch (NumberFormatException nfe) {

                    }
                }
            }
        }

        if (StringUtils.isEmpty(host)) {
            host = envHost;
        }

        if (port <= 0) {
            port = envPort;
        }

        return new P4ScmProviderRepository(host, port, path, user, password);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#requiresEditMode()
     */
    public boolean requiresEditMode() {
        return true;
    }

    /**
     * @see org.apache.maven.scm.provider.ScmProvider#getScmType()
     */
    public String getScmType() {
        return "p4";
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#add(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected AddScmResult add(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4AddCommand command = new P4AddCommand();
        command.setLogger(getLogger());
        return (AddScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#blame(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected BlameScmResult blame(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4BlameCommand command = new P4BlameCommand();
        command.setLogger(getLogger());
        return (BlameScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#branch(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected BranchScmResult branch(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4BranchCommand command = new P4BranchCommand();
        command.setLogger(getLogger());
        return (BranchScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#checkin(org.apache.
     *      maven.scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected CheckInScmResult checkin(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4CheckInCommand command = new P4CheckInCommand();
        command.setLogger(getLogger());
        return (CheckInScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#checkout(org.apache
     *      .maven.scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected CheckOutScmResult checkout(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4CheckOutCommand command = new P4CheckOutCommand();
        command.setLogger(getLogger());
        return (CheckOutScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#changelog(org.apache
     *      .maven.scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected ChangeLogScmResult changelog(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters parameters)
            throws ScmException {
        P4ChangeLogCommand command = new P4ChangeLogCommand();
        command.setLogger(getLogger());
        return (ChangeLogScmResult) command.execute(repository, fileSet,
                parameters);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#diff(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected DiffScmResult diff(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4DiffCommand command = new P4DiffCommand();
        command.setLogger(getLogger());
        return (DiffScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#edit(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected EditScmResult edit(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4EditCommand command = new P4EditCommand();
        command.setLogger(getLogger());
        return (EditScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#export(org.apache
     *      .maven.scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected ExportScmResult export(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4ExportCommand command = new P4ExportCommand();
        command.setLogger(getLogger());
        return (ExportScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#list(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    public ListScmResult list(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4ListCommand command = new P4ListCommand();
        command.setLogger(getLogger());
        return (ListScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#login(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected LoginScmResult login(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4LoginCommand command = new P4LoginCommand();
        command.setLogger(getLogger());
        return (LoginScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#remove(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected RemoveScmResult remove(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4RemoveCommand command = new P4RemoveCommand();
        command.setLogger(getLogger());
        return (RemoveScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#status(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected StatusScmResult status(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4StatusCommand command = new P4StatusCommand();
        command.setLogger(getLogger());
        return (StatusScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#tag(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected TagScmResult tag(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4TagCommand command = new P4TagCommand();
        command.setLogger(getLogger());
        return (TagScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#unedit(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected UnEditScmResult unedit(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4UnEditCommand command = new P4UnEditCommand();
        command.setLogger(getLogger());
        return (UnEditScmResult) command.execute(repository, fileSet, params);
    }

    /**
     * @see org.apache.maven.scm.provider.AbstractScmProvider#update(org.apache.maven
     *      .scm.provider.ScmProviderRepository,
     *      org.apache.maven.scm.ScmFileSet,
     *      org.apache.maven.scm.CommandParameters)
     */
    protected UpdateScmResult update(ScmProviderRepository repository,
            ScmFileSet fileSet, CommandParameters params) throws ScmException {
        P4UpdateCommand command = new P4UpdateCommand();
        command.setLogger(getLogger());
        return (UpdateScmResult) command.execute(repository, fileSet, params);
    }
}
